//var crypto = require('crypto');
var rb = require('crypto').randomBytes;

function bitsToDouble(bits) {
    return new Buffer(
        bits.split(' ').map( function (bits) {return parseInt( bits, 2 )} )
    ).readDoubleBE(0)
}

function bitsToUInt32(bits) {
    return new Buffer(
        bits.split(' ').map( function (bits) {return parseInt( bits, 2 )} )
    ).readUInt32BE(0)
}

// FASTEST
// 1e6 in 4580ms
// Returns a string of 14 random decimal digits.
function getRandDouble14() {
    var buf, n;

    // Probability of repeat = 3599627370497 / 4503599627370496 =~ 0.000799 (i.e. 1 in 1251).
    // Range of n is 4503599627370496 (mantissa all zeros) to 9007199254740991 (mantissa all ones).
    // 4503599627370496 - 3599627370496 = 4500000000000000
    // 9007199254740991 - 3599627370496 = 9003599627370495
    // 9003599627370495 - 4503599627370496 = 4499999999999999
    do {
        buf = rb(8);
        // Set the sign and exponent bits to get a positive integer between 2^52 and 2^53.
        buf[0] = 0x43;                      // 01000011
        buf[1] = buf[1] & 0xf | 0x30;       // 0011????
        n = buf.readDoubleBE(0);
    } while ( n > 9003599627370495 );

    /*
      If we subtract 4503599627370496 we get n in range 0 to 4499999999999999,
      so if we subtract 1e15 less than that we get n in range 1000000000000000
      to 5499999999999999. Return the last 14 digits as a string.
     */
    return ( n - 3503599627370496 ).toString().slice(2);
}

// 1e6 in 4800ms
// Returns a string of 15 random decimal digits.
function getRandDouble15() {
    var buf, n;

    // Probability of repeat = ( 9007199254740991 - 8503599627370495 ) / 4503599627370496 =~ 0.111 (i.e. 1 in 9).
    // Range of n is 4503599627370496 (mantissa all zeros) to 9007199254740991 (mantissa all ones).
    // 9007199254740991 - 8503599627370495 = 503599627370496
    // 8503599627370495 - 4503599627370496 = 3999999999999999
    do {
        buf = rb(8);
        // Set the sign and exponent bits to get a positive integer between 2^52 and 2^53.
        buf[0] = 0x43;                      // 01000011
        buf[1] = buf[1] & 0xf | 0x30;       // 0011????
        n = buf.readDoubleBE(0);
    } while ( n > 8503599627370495 );

    /*
      If we subtract 4503599627370496 we get n in range 0 to 3999999999999999,
      so if we subtract 1e15 less than that we get n in range 1000000000000000
      to 4999999999999999.
     */
    return ( n - 3503599627370496 ).toString().slice(1);
}

// 1e6 in 5500ms
function getRand5bytes() {
    var n;

    // 0xffffffffff = 1099511627775
    // Probability of repeat = 99511627776 / 1099511627776 =~ 0.09 (i.e. 1 in 11).
    do {
        n = parseInt( rb(5).toString('hex'), 16 );
    } while ( n >= 1e12 );

    // Add 1e12 so n is in range 1000000000000 to 1999999999999 inclusive.
    // Return a string of 12 random decimal digits.
    return ( n + 1e12 ).toString().slice(1);
}

// 1e6 in 4300ms
// 31 bits = 2^31 - 1 = 2147483647, so with 0 there are 2^31 possible values when using 31 bits.
function getRandomDigits() {
    var buf, n;

    // Probability of repetition = 147483648 / 2147483648 =~ 0.0686 (i.e. 1 in 14.5).
    do {
        buf = rb(4);

        // 0 >= n < 2147483648
        n = ( ( buf[3] & 0x7f ) << 24 ) + ( buf[2] << 16 ) + ( buf[1] << 8 ) + buf[0];
    } while ( n >= 2e9 );

    // Add 1e9 so 1000000000 >= n <= 2999999999 and return 9 digits.
    return ( n + 1e9 ).toString().slice(1);
}


function getSixRandomDigits() {
    var buf, n;

    // Probability of repetition = 777216 / 16777216 =~ 0.046 (i.e. 1 in 22).
    do {
        buf = crypto.randomBytes(3);

        // 0 >= n < 16777216
        // n = parseInt( buf.toString('hex'), 16 );
        n = ( buf[0] << 16 ) + ( buf[1] << 8 ) + buf[2];
    } while ( n >= 1.6e7 );

    // Add 1e7 so 10000000 >= n <= 25999999 and return the last 6 digits.
    return ( n + 1e7 ).toString().slice(2);
}

/************************/

function getRandNode(one) {
    var buf, n;

    if (one) {

        do {
            n = rb(1)[0];
        } while ( n > 249 );
        n = n % 10 + '';
    } else {

        do {
            buf = rb(8);
            buf[0] = 0x43;
            buf[1] = buf[1] & 0xf | 0x30;
            n = buf.readDoubleBE(0);
        } while ( n > 9003599627370495 );
        n = ( n - 3503599627370496 + '' ).slice(2);
    }

    return n;
}

// getRandNode2, getRandNode3 and getRandNode4 are identical.

function getRandNode2(max) {
    var buf, limit, n, str;

    if (max) {
       limit = ( 256 / max | 0 ) * max - 1;

        do {
            n = rb(1)[0];
        } while ( n > limit );
        str = n % max + '';
    } else {

        do {
            buf = rb(8);
            buf[0] = 0x43;
            buf[1] = buf[1] & 0xf | 0x30;
            n = buf.readDoubleBE(0);
        } while ( n > 9003599627370495 );
        str = ( n - 3503599627370496 + '' ).slice(2);
    }

    return str;
}

function getRandNode3(max) {
     var buf, n;

    if (max) {
        n = rb(1)[0];
        return n > ( 256 / max | 0 ) * max - 1 ? getRandNode3(max) : n % max + '';
    } else {
        buf = rb(8);
        buf[0] = 0x43;
        buf[1] = buf[1] & 0xf | 0x30;
        n = buf.readDoubleBE(0);
        return n > 9003599627370495 ? getRandNode3(max) : ( n - 3503599627370496 + '' ).slice(2);
    }
}

function getRandNode4(max) {
    var buf, n;

    return max ? ( n = rb(1)[0], n > ( 256 / max | 0 ) * max - 1 ? getRandNode4(max) : n % max + '' ) :
    ( buf = rb(8), buf[0] = 0x43, buf[1] = buf[1] & 0xf | 0x30, n = buf.readDoubleBE(0),
      n > 9003599627370495 ? getRandNode4(max) : ( n - 3503599627370496 + '' ).slice(2) );
}



// 1e6 in 3200ms
// Recalls itself about 1 in 43.
function nextDigit() {
    var n = rb(1)[0];
    //return n < 250 ? n * 0.04 | 0 : nextDigit();
    return n < 250 ? n % 10 : nextDigit();
}

// 1e6 in 20ms
function next() {
    return Math.random() * 10 | 0;
}

var start = +new Date();

/*
*/
for ( var i = 0; i < 1e2; i++ ) {
    console.log( getRandNode3(2) );
    //getRandNode2();
    //getRandy();
    //nextDigit();
    //console.log( getSixRandomDigits() );
    //getRandomDigits();
    //getRandDouble14();
}

/*
getRandomDouble14: 3570 ms
getRandomDouble15: 3640 ms
getRand5bytes: 5250 ms
getRandomDigits: 5150 ms
var str = '';
while ( str.length < 1e7 ) {
    //str += getRandDouble15();
    //str += getRandomDigits();
    str += getRand();
}
*/

console.log('\nTime taken: ' + (+new Date() - start) + ' ms \n');



/*
console.log(
    //bitsToUInt32( '00000000 00000000 00000000 00000000' )    // 0
    //bitsToUInt32( '11111111 11111111 11111111 11111111' )    // 4294967295
    //bitsToUInt32( '11101110 01101011 00101000 00000000' )      // 4000000000
    bitsToUInt32( '11101110 00000000 00000000 00000000' )      // 4000000000
  //bitsToDouble( '01000011 00110000 00000000 00000000 00000000 00000000 00000000 00000000' )
  //bitsToDouble( '01000011 00111111 11111001 01110011 11001010 11111010 10000000 00000000' )    // 9000000000000000
.toString() );
*/

// Subtract 1023 from exponent to get actual exponent.
// 0 00000000000 1.00000000000000000000000000000000000000000000000000000
// s eeeeeeeeeee   mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm

// -1^s
// eeeeeeeeeee 2^10 = 1024 ... 2^0 = 1
// 1.mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm 1 * 2^0 ... + 1 * 2^-52

// seeeeeee eeeemmmm mmmmmmmm mmmmmmmm mmmmmmmm mmmmmmmm mmmmmmmm mmmmmmmm
// 00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000    //  0
// 10000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000    // -0
// 00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000001    //  5e-324
// 10000000 00000000 00000000 00000000 00000000 00000000 00000000 00000001    // -5e-324
// 01111111 11101111 11111111 11111111 11111111 11111111 11111111 11111111    //  1.7976931348623157e+308
// 11111111 11101111 11111111 11111111 11111111 11111111 11111111 11111111    // -1.7976931348623157e+308

// Between 2^52 = 4,503,599,627,370,496 and 2^53 = 9,007,199,254,740,992 the representable numbers are exactly the integers.
// 01000011 00110000 00000000 00000000 00000000 00000000 00000000 00000000    // 4503599627370496
//  1000011 0011 = 1075 (1075-1023 = 52)  2^52 = 4503599627370496
// 01000011 00111111 11111111 11111111 11111111 11111111 11111111 11111111    // 9007199254740991
//  1000011 0011 = 1075 (1075-1023 = 52)
// 01000011 01000000 00000000 00000000 00000000 00000000 00000000 00000000    // 9007199254740992
//  1000011 0100 = 1076 (1076-1023 = 53)  2^53 = 9007199254740992
// 01000011 00111111 11111001 01110011 11001010 11111010 10000000 00000000    // 9000000000000000


// e's all 1 and m's all 0 is +-Infinity
// 01111111 11110000 00000000 00000000 00000000 00000000 00000000 00000000    //  Infinity
// 11111111 11110000 00000000 00000000 00000000 00000000 00000000 00000000    // -Infinity

// e's all 1 and m's not all 0 is NaN
// 11111111 11111000 00000000 00000000 00000000 00000000 00000000 00000000    // NaN
// 11111111 11111111 11111111 11111111 11111111 11111111 11111111 11111111    // NaN

// 00111111 11110000 00000000 00000000 00000000 00000000 00000000 00000000    //  1
// 10111111 11110000 00000000 00000000 00000000 00000000 00000000 00000000    // -1

// 01000000 00000000 00000000 00000000 00000000 00000000 00000000 00000001    // 2.0000000000000004
// -1^0 * 2^(1024-1023) * ( 1 * 2^0 + 1 * 2^-52 )
//    1 * 2             * 1.000000000000000222044604925...                    // 2.00000000000000044408920985006261...

// 01000000 00000000 00000000 00000000 00000000 00000000 00000000 10000000    // 2.000000000000057
// -1^0 * 2^(1024-1023) * ( 1 * 2^0 + 1 * 2^-45 )
//    1 * 2             * 1.00000000000002842170943040400743...               // 2.00000000000005684341886080801486...

// 01111111 1110000 00000000 00000000 00000000 00000000 00000000 00000011     // 8.988465674311586e+307
// 11111111110 = 2046
// -1^0 * 2^(2046-1023) * ( 1 * 2^0 + 1 * 2^-51 + 1 * 2^-52 )
//    1 * 8.988...e+307 * 1.000000000000000666133814775093...                 // 8.988465674311585506...e+307


